"""ACC (Autodesk Construction Cloud) client module."""
from __future__ import annotations

from collections.abc import Awaitable, Callable
from typing import Optional, TYPE_CHECKING

import httpx
from autodesk_common_httpclient import HttpClientFactory

from autodesk_acc.generated_code.base_a_c_cclient import BaseACCclient

if TYPE_CHECKING:
    from autodesk_acc.generated_code.bim360.clash.v3.v3_request_builder import (
        V3RequestBuilder as ClashV3RequestBuilder,
    )
    from autodesk_acc.generated_code.bim360.docs.v1.v1_request_builder import (
        V1RequestBuilder as DocsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.bim360.modelset.v3.v3_request_builder import (
        V3RequestBuilder as ModelSetV3RequestBuilder,
    )
    from autodesk_acc.generated_code.bim360.relationship.v2.v2_request_builder import (
        V2RequestBuilder as RelationshipV2RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.admin.v1.v1_request_builder import (
        V1RequestBuilder as AdminV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.assets.assets_request_builder import (
        AssetsRequestBuilder,
    )
    from autodesk_acc.generated_code.construction.autospecs.v1.v1_request_builder import (
        V1RequestBuilder as AutoSpecsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.files.v1.v1_request_builder import (
        V1RequestBuilder as FilesV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.forms.v1.v1_request_builder import (
        V1RequestBuilder as FormsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.index.v2.v2_request_builder import (
        V2RequestBuilder as IndexV2RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.issues.v1.v1_request_builder import (
        V1RequestBuilder as IssuesV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.locations.v2.v2_request_builder import (
        V2RequestBuilder as LocationsV2RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.packages.v1.v1_request_builder import (
        V1RequestBuilder as PackagesV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.photos.v1.v1_request_builder import (
        V1RequestBuilder as PhotosV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.rcm.v1.v1_request_builder import (
        V1RequestBuilder as RcmV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.reviews.v1.v1_request_builder import (
        V1RequestBuilder as ReviewsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.rfis.v3.v3_request_builder import (
        V3RequestBuilder as RfisV3RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.sheets.v1.v1_request_builder import (
        V1RequestBuilder as SheetsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.submittals.v2.v2_request_builder import (
        V2RequestBuilder as SubmittalsV2RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.takeoff.v1.v1_request_builder import (
        V1RequestBuilder as TakeoffV1RequestBuilder,
    )
    from autodesk_acc.generated_code.construction.transmittals.v1.v1_request_builder import (
        V1RequestBuilder as TransmittalsV1RequestBuilder,
    )
    from autodesk_acc.generated_code.cost.v1.v1_request_builder import (
        V1RequestBuilder as CostV1RequestBuilder,
    )
    from autodesk_acc.generated_code.data_connector.v1.v1_request_builder import (
        V1RequestBuilder as DataConnectorV1RequestBuilder,
    )
    from autodesk_acc.generated_code.hq.v1.accounts.accounts_request_builder import (
        AccountsRequestBuilder,
    )


class ACCClient:
    """High-level Autodesk Construction Cloud client.

    Provides convenient shortcut properties for all ACC API endpoint groups.

    Args:
        get_access_token: Async function that returns a valid access token.
        http_client: Optional httpx.AsyncClient to override the default HTTP client.
    """

    def __init__(
        self,
        get_access_token: Callable[[], Awaitable[str]],
        http_client: Optional[httpx.AsyncClient] = None,
    ) -> None:
        adapter = HttpClientFactory.create_adapter(get_access_token, http_client)
        self._api = BaseACCclient(adapter)

    @property
    def api(self) -> BaseACCclient:
        """The full generated ACC API client (base path https://developer.api.autodesk.com)."""
        return self._api

    # ── Account Admin shortcuts ──────────────────────────────────────────

    @property
    def accounts(self) -> AccountsRequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/hq/v1/accounts/*"""
        return self._api.hq.v1.accounts

    @property
    def admin(self) -> AdminV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/admin/v1/*"""
        return self._api.construction.admin.v1

    # ── Construction shortcuts ───────────────────────────────────────────

    @property
    def assets(self) -> AssetsRequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/assets/*"""
        return self._api.construction.assets

    @property
    def auto_specs(self) -> AutoSpecsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/autospecs/v1/*"""
        return self._api.construction.autospecs.v1

    @property
    def files(self) -> FilesV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/files/v1/*"""
        return self._api.construction.files.v1

    @property
    def forms(self) -> FormsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/forms/v1/*"""
        return self._api.construction.forms.v1

    @property
    def index(self) -> IndexV2RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/index/v2/*"""
        return self._api.construction.index.v2

    @property
    def issues(self) -> IssuesV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/issues/v1/*"""
        return self._api.construction.issues.v1

    @property
    def locations(self) -> LocationsV2RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/locations/v2/*"""
        return self._api.construction.locations.v2

    @property
    def packages(self) -> PackagesV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/packages/v1/*"""
        return self._api.construction.packages.v1

    @property
    def photos(self) -> PhotosV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/photos/v1/*"""
        return self._api.construction.photos.v1

    @property
    def rcm(self) -> RcmV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/rcm/v1/*"""
        return self._api.construction.rcm.v1

    @property
    def reviews(self) -> ReviewsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/reviews/v1/*"""
        return self._api.construction.reviews.v1

    @property
    def rfis(self) -> RfisV3RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/rfis/v3/*"""
        return self._api.construction.rfis.v3

    @property
    def sheets(self) -> SheetsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/sheets/v1/*"""
        return self._api.construction.sheets.v1

    @property
    def submittals(self) -> SubmittalsV2RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/submittals/v2/*"""
        return self._api.construction.submittals.v2

    @property
    def takeoff(self) -> TakeoffV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/takeoff/v1/*"""
        return self._api.construction.takeoff.v1

    @property
    def transmittals(self) -> TransmittalsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/construction/transmittals/v1/*"""
        return self._api.construction.transmittals.v1

    # ── BIM 360 shortcuts ────────────────────────────────────────────────

    @property
    def clash(self) -> ClashV3RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/bim360/clash/v3/*"""
        return self._api.bim360.clash.v3

    @property
    def docs(self) -> DocsV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/bim360/docs/v1/*"""
        return self._api.bim360.docs.v1

    @property
    def model_set(self) -> ModelSetV3RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/bim360/modelset/v3/*"""
        return self._api.bim360.modelset.v3

    @property
    def projects(self) -> RelationshipV2RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/bim360/relationship/v2/*"""
        return self._api.bim360.relationship.v2

    @property
    def relationships(self) -> RelationshipV2RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/bim360/relationship/v2/*"""
        return self._api.bim360.relationship.v2

    # ── Cost shortcuts ───────────────────────────────────────────────────

    @property
    def cost(self) -> CostV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/cost/v1/*"""
        return self._api.cost.v1

    # ── Data Connector shortcuts ─────────────────────────────────────────

    @property
    def data_connector(self) -> DataConnectorV1RequestBuilder:
        """Shortcut to https://developer.api.autodesk.com/dataconnector/v1/*"""
        return self._api.data_connector.v1



from enum import Enum

class PaymentsGetResponse_status(str, Enum):
    Draft = "draft",
    PendingInput = "pendingInput",
    Submitted = "submitted",
    Revise = "revise",
    Accepted = "accepted",
    Approved = "approved",
    Paid = "paid",




from enum import Enum

class GetAssociationTypeQueryParameterType(str, Enum):
    Budget = "Budget",
    Contract = "Contract",
    CostItem = "CostItem",
    FormInstance = "FormInstance",
    Payment = "Payment",
    BudgetPayment = "BudgetPayment",
    Expense = "Expense",
    OCO = "OCO",
    SCO = "SCO",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ThumbnailsBatchGetPostResponse_results(Parsable):
    # The ID of the review sheet the thumbnail belongs to.
    review_sheet_id: Optional[str] = None
    # The URL of the thumbnail. It is an AWS S3 signed URL and will expire in 1 hour.
    signed_url: Optional[str] = None
    # The width and height of the thumbnail.
    size: Optional[list[int]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ThumbnailsBatchGetPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ThumbnailsBatchGetPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ThumbnailsBatchGetPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "reviewSheetId": lambda n : setattr(self, 'review_sheet_id', n.get_str_value()),
            "signedUrl": lambda n : setattr(self, 'signed_url', n.get_str_value()),
            "size": lambda n : setattr(self, 'size', n.get_collection_of_primitive_values(int)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("reviewSheetId", self.review_sheet_id)
        writer.write_str_value("signedUrl", self.signed_url)
        writer.write_collection_of_primitive_values("size", self.size)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_response_patch_response_rfi_architects import WithResponsePatchResponse_rfi_architects
    from .with_response_patch_response_rfi_assigned_to import WithResponsePatchResponse_rfi_assignedTo
    from .with_response_patch_response_rfi_co_reviewers import WithResponsePatchResponse_rfi_coReviewers
    from .with_response_patch_response_rfi_custom_attributes import WithResponsePatchResponse_rfi_customAttributes
    from .with_response_patch_response_rfi_draft_responses import WithResponsePatchResponse_rfi_draftResponses
    from .with_response_patch_response_rfi_official_response_actors import WithResponsePatchResponse_rfi_officialResponseActors
    from .with_response_patch_response_rfi_official_response_status import WithResponsePatchResponse_rfi_officialResponseStatus
    from .with_response_patch_response_rfi_permitted_actions import WithResponsePatchResponse_rfi_permittedActions
    from .with_response_patch_response_rfi_previous_status import WithResponsePatchResponse_rfi_previousStatus
    from .with_response_patch_response_rfi_responses import WithResponsePatchResponse_rfi_responses
    from .with_response_patch_response_rfi_reviewers import WithResponsePatchResponse_rfi_reviewers
    from .with_response_patch_response_rfi_status import WithResponsePatchResponse_rfi_status
    from .with_response_patch_response_rfi_watchers import WithResponsePatchResponse_rfi_watchers
    from .with_response_patch_response_rfi_workflow_type import WithResponsePatchResponse_rfi_workflowType

@dataclass
class WithResponsePatchResponse_rfi(Parsable):
    # The date and time when the official response to the RFI was submitted, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).Empty when creating an RFI.
    answered_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who submitted the official response to the RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.Empty when creating an RFI.
    answered_by: Optional[str] = None
    # The list of architect users associated with the RFI.
    architects: Optional[list[WithResponsePatchResponse_rfi_architects]] = None
    # The list of users assigned to the RFI.
    assigned_to: Optional[list[WithResponsePatchResponse_rfi_assignedTo]] = None
    # Not relevant
    bridge_sync_outdated: Optional[bool] = None
    # Not relevant
    bridged_source: Optional[bool] = None
    # Not relevant
    bridged_target: Optional[bool] = None
    # A list of predefined categories to assign to the RFI.Categories help group RFIs for filtering and reporting. Each value must match a category configured in the project’s RFI settings. Categories are case-sensitive and project-specific.RFI categories are configured in Project Admin and may differ between projects. Call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ to retrieve the allowed values for this field.Some possible values: ``Code Compliance``, ``Constructability``, ``Design Coordination``, ``Documentation Conflict``, ``Documentation Incomplete``, ``Field condition``, ``Other``.
    category: Optional[list[str]] = None
    # The date and time when the RFI was closed, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).
    closed_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who closed the RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    closed_by: Optional[str] = None
    # A list of reviewers assigned to the RFI. Each entry may represent a user, role, or company.
    co_reviewers: Optional[list[WithResponsePatchResponse_rfi_coReviewers]] = None
    # The number of comments associated with the RFI.
    comments_count: Optional[int] = None
    # The Autodesk ID of the user designated as the Construction Manager for this RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    construction_manager_id: Optional[str] = None
    # The ID of the container.
    container_id: Optional[str] = None
    # The default cost impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To check whether cost impact options are enabled and to retrieve the default value, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    cost_impact: Optional[str] = None
    # The date and time when the RFI was created, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    created_by: Optional[str] = None
    # A list of custom attributes associated with the RFI.
    custom_attributes: Optional[list[WithResponsePatchResponse_rfi_customAttributes]] = None
    # The user-defined identifier of the RFI.
    custom_identifier: Optional[str] = None
    # The discipline associated with the RFI. To retrieve the supported values for the current project, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.Some possible values: ``Building Management System``, ``Electrical Substation``, ``Security``, ``Audio Visual``, ``Food Service``, ``Fire Alarm``, ``Power Systems``, ``Design Systems Integrator``, ``Signage``, ``Pathways``, ``Cabling``, ``Networks``, ``Distributed Antenna System``, ``Lighting``, ``Vertical Transportation``, ``Roofing``, ``Architectural``, ``Civil/Site``, ``Concrete``, ``Electrical``, ``Exterior Envelope``, ``Fire Protection``, ``Interior/Finishes``, ``Landscaping``, ``Masonry``, ``Mechanical``, ``Plumbing``, ``Structural``, ``Other``.'
    discipline: Optional[list[str]] = None
    # A list of draft responses associated with this RFI.Always empty when creating an RFI.
    draft_responses: Optional[list[WithResponsePatchResponse_rfi_draftResponses]] = None
    # The date and time by which a response to the RFI is expected, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).
    due_date: Optional[str] = None
    # The system-generated ID of the RFI.
    id: Optional[str] = None
    # The default text for the Location field when creating a new RFI.Note that the API does not auto-populate this value.Clients are responsible for applying the default if desired.To retrieve the default value configured for this field, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.
    location_description: Optional[str] = None
    # A list of predefined location IDs associated with the RFI, based on the project’s Location Breakdown Structure (LBS). To get more information about the locations, call `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.
    locations: Optional[list[str]] = None
    # The Autodesk ID of the user designated as the RFI Manager.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    manager_id: Optional[str] = None
    # The max amount of assignees permitted base on the RFIs current status.
    max_assignees: Optional[int] = None
    # The text of the official response submitted for the RFI.Always empty when creating an RFI.
    official_response: Optional[str] = None
    # The list of users who contributed to the official response.Always empty when creating an RFI.
    official_response_actors: Optional[list[WithResponsePatchResponse_rfi_officialResponseActors]] = None
    # ``true``: the RFI Manager is allowed to edit the official response after submission.``false``: editing the official response is disabled. (default).
    official_response_edit_by_manager_state: Optional[bool] = None
    # The status of the official response to the RFI.Possible values: ``unanswered``, ``answered``.Always ``unanswered`` when creating an RFI.
    official_response_status: Optional[WithResponsePatchResponse_rfi_officialResponseStatus] = None
    # The list of actions that are permitted for the user.
    permitted_actions: Optional[WithResponsePatchResponse_rfi_permittedActions] = None
    # The previous status of the RFI, if one exists.This field is omitted if the RFI has no prior status (e.g., when newly created).- For single-reviewer workflows (US):Possible values:``draft``, ``submitted``, ``open``, ``answered``, ``rejected``, ``closed``, ``void``.- For multi-reviewer workflows (EMEA):Possible values:``draft``, ``submitted``, ``openRev1`` (manager), ``openRev2`` (reviewers), ``answeredRev1``, ``answeredManager``, ``closed``, ``void``.To determine the workflow type, call `GET users/me </en/docs/acc/v1/reference/http/rfis-users-me-GET/>`_ and check the ``workflowType`` value.For details on RFI workflows in the ACC UI, see `About RFI Workflows – Autodesk Help <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Workflow_Setup>`_.
    previous_status: Optional[WithResponsePatchResponse_rfi_previousStatus] = None
    # The default priority for new RFIs of this type.The available priority values are configured in Project Admin.If no default is set, this field is ``null``.Note that the API does not auto-populate this value when creating an RFI.Clients are responsible for applying the default if desired.The valid priority options can be retrieved by calling `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>_. Some possible values: ``null``, ``High``, ``Normal``, ``Low``.
    priority: Optional[str] = None
    # The Autodesk ID of the project the RFI belongs to.
    project_id: Optional[str] = None
    # The question submitted in the RFI.
    question: Optional[str] = None
    # A user-provided text reference related to the RFI, such as a model number or spec reference, typically used when the RFI was created in another system.Max length: 20
    reference: Optional[str] = None
    # The date and time when the RFI was officially responded to, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).
    responded_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who submitted the official response to the RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    responded_by: Optional[str] = None
    # A list of responses associated with this RFI.Always empty when creating an RFI.
    responses: Optional[list[WithResponsePatchResponse_rfi_responses]] = None
    # The list of users assigned to review the RFI before it is closed.
    reviewers: Optional[list[WithResponsePatchResponse_rfi_reviewers]] = None
    # The ID of the default RFI type assigned to the project. This is the unique identifier of the RFI type that will be selected by default when creating a new RFI.
    rfi_type_id: Optional[UUID] = None
    # The default schedule impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To verify whether schedule impact tracking is enabled for the project and what the default value is, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    schedule_impact: Optional[str] = None
    # The current status of the RFI. Available values depend on the RFI’s workflow type:- For single-reviewer workflows (US):Possible values:``draft``, ``submitted``, ``open``, ``answered``, ``rejected``, ``closed``, ``void``.- For multi-reviewer workflows (EMEA):Possible values:``draft``, ``submitted``, ``openRev1`` (manager), ``openRev2`` (reviewers), ``answeredRev1``, ``answeredManager``, ``closed``, ``void``.To determine the workflow type, call `GET users/me </en/docs/acc/v1/reference/http/rfis-users-me-GET/>`_ and check the ``workflowType`` value.For details on RFI workflows in the ACC UI, see `About RFI  Workflows – Autodesk Help <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Workflow_Setup>`_.
    status: Optional[WithResponsePatchResponse_rfi_status] = None
    # A suggested answer for the RFI, typically entered by the assignee before submission of the official response.
    suggested_answer: Optional[str] = None
    # Not relevant
    sync_token: Optional[str] = None
    # Not relevant
    sync_version: Optional[float] = None
    # The title of the RFI.
    title: Optional[str] = None
    # The date and time when the RFI was last updated, in ISO 8601 format (``YYYY-MM-DDThh:mm:ss.sZ``).
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the RFI.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    updated_by: Optional[str] = None
    # The URN of the virtual folder created for the RFI.This folder stores all attachments related to the RFI.The ``virtualFolderUrn`` is required when uploading attachments to an RFI. See the `Upload Attachment </en/docs/acc/v1/tutorials/files/upload-document-s3/>`_ tutorial for more details.
    virtual_folder_urn: Optional[str] = None
    # A list of watchers who are notified about changes to the RFI. Each entry may represent a user, role, or company.
    watchers: Optional[list[WithResponsePatchResponse_rfi_watchers]] = None
    # The workflow type assigned to the RFI, which determines the allowed status transitions and the review path.Possible values:- ``US``: Single-reviewer workflow- ``EU``: Multi-reviewer workflowThis value affects how statuses like ``submitted``, ``openRev1``, or ``answeredManager`` behave.For status definitions, see the ``status`` and ``previousStatus`` fields.
    workflow_type: Optional[WithResponsePatchResponse_rfi_workflowType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchResponse_rfi:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchResponse_rfi
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchResponse_rfi()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_response_rfi_architects import WithResponsePatchResponse_rfi_architects
        from .with_response_patch_response_rfi_assigned_to import WithResponsePatchResponse_rfi_assignedTo
        from .with_response_patch_response_rfi_co_reviewers import WithResponsePatchResponse_rfi_coReviewers
        from .with_response_patch_response_rfi_custom_attributes import WithResponsePatchResponse_rfi_customAttributes
        from .with_response_patch_response_rfi_draft_responses import WithResponsePatchResponse_rfi_draftResponses
        from .with_response_patch_response_rfi_official_response_actors import WithResponsePatchResponse_rfi_officialResponseActors
        from .with_response_patch_response_rfi_official_response_status import WithResponsePatchResponse_rfi_officialResponseStatus
        from .with_response_patch_response_rfi_permitted_actions import WithResponsePatchResponse_rfi_permittedActions
        from .with_response_patch_response_rfi_previous_status import WithResponsePatchResponse_rfi_previousStatus
        from .with_response_patch_response_rfi_responses import WithResponsePatchResponse_rfi_responses
        from .with_response_patch_response_rfi_reviewers import WithResponsePatchResponse_rfi_reviewers
        from .with_response_patch_response_rfi_status import WithResponsePatchResponse_rfi_status
        from .with_response_patch_response_rfi_watchers import WithResponsePatchResponse_rfi_watchers
        from .with_response_patch_response_rfi_workflow_type import WithResponsePatchResponse_rfi_workflowType

        from .with_response_patch_response_rfi_architects import WithResponsePatchResponse_rfi_architects
        from .with_response_patch_response_rfi_assigned_to import WithResponsePatchResponse_rfi_assignedTo
        from .with_response_patch_response_rfi_co_reviewers import WithResponsePatchResponse_rfi_coReviewers
        from .with_response_patch_response_rfi_custom_attributes import WithResponsePatchResponse_rfi_customAttributes
        from .with_response_patch_response_rfi_draft_responses import WithResponsePatchResponse_rfi_draftResponses
        from .with_response_patch_response_rfi_official_response_actors import WithResponsePatchResponse_rfi_officialResponseActors
        from .with_response_patch_response_rfi_official_response_status import WithResponsePatchResponse_rfi_officialResponseStatus
        from .with_response_patch_response_rfi_permitted_actions import WithResponsePatchResponse_rfi_permittedActions
        from .with_response_patch_response_rfi_previous_status import WithResponsePatchResponse_rfi_previousStatus
        from .with_response_patch_response_rfi_responses import WithResponsePatchResponse_rfi_responses
        from .with_response_patch_response_rfi_reviewers import WithResponsePatchResponse_rfi_reviewers
        from .with_response_patch_response_rfi_status import WithResponsePatchResponse_rfi_status
        from .with_response_patch_response_rfi_watchers import WithResponsePatchResponse_rfi_watchers
        from .with_response_patch_response_rfi_workflow_type import WithResponsePatchResponse_rfi_workflowType

        fields: dict[str, Callable[[Any], None]] = {
            "answeredAt": lambda n : setattr(self, 'answered_at', n.get_datetime_value()),
            "answeredBy": lambda n : setattr(self, 'answered_by', n.get_str_value()),
            "architects": lambda n : setattr(self, 'architects', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_architects)),
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_assignedTo)),
            "bridgeSyncOutdated": lambda n : setattr(self, 'bridge_sync_outdated', n.get_bool_value()),
            "bridgedSource": lambda n : setattr(self, 'bridged_source', n.get_bool_value()),
            "bridgedTarget": lambda n : setattr(self, 'bridged_target', n.get_bool_value()),
            "category": lambda n : setattr(self, 'category', n.get_collection_of_primitive_values(str)),
            "closedAt": lambda n : setattr(self, 'closed_at', n.get_datetime_value()),
            "closedBy": lambda n : setattr(self, 'closed_by', n.get_str_value()),
            "coReviewers": lambda n : setattr(self, 'co_reviewers', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_coReviewers)),
            "commentsCount": lambda n : setattr(self, 'comments_count', n.get_int_value()),
            "constructionManagerId": lambda n : setattr(self, 'construction_manager_id', n.get_str_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_str_value()),
            "costImpact": lambda n : setattr(self, 'cost_impact', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_customAttributes)),
            "customIdentifier": lambda n : setattr(self, 'custom_identifier', n.get_str_value()),
            "discipline": lambda n : setattr(self, 'discipline', n.get_collection_of_primitive_values(str)),
            "draftResponses": lambda n : setattr(self, 'draft_responses', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_draftResponses)),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "locationDescription": lambda n : setattr(self, 'location_description', n.get_str_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "managerId": lambda n : setattr(self, 'manager_id', n.get_str_value()),
            "maxAssignees": lambda n : setattr(self, 'max_assignees', n.get_int_value()),
            "officialResponse": lambda n : setattr(self, 'official_response', n.get_str_value()),
            "officialResponseActors": lambda n : setattr(self, 'official_response_actors', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_officialResponseActors)),
            "officialResponseEditByManagerState": lambda n : setattr(self, 'official_response_edit_by_manager_state', n.get_bool_value()),
            "officialResponseStatus": lambda n : setattr(self, 'official_response_status', n.get_enum_value(WithResponsePatchResponse_rfi_officialResponseStatus)),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_object_value(WithResponsePatchResponse_rfi_permittedActions)),
            "previousStatus": lambda n : setattr(self, 'previous_status', n.get_enum_value(WithResponsePatchResponse_rfi_previousStatus)),
            "priority": lambda n : setattr(self, 'priority', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "question": lambda n : setattr(self, 'question', n.get_str_value()),
            "reference": lambda n : setattr(self, 'reference', n.get_str_value()),
            "respondedAt": lambda n : setattr(self, 'responded_at', n.get_datetime_value()),
            "respondedBy": lambda n : setattr(self, 'responded_by', n.get_str_value()),
            "responses": lambda n : setattr(self, 'responses', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_responses)),
            "reviewers": lambda n : setattr(self, 'reviewers', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_reviewers)),
            "rfiTypeId": lambda n : setattr(self, 'rfi_type_id', n.get_uuid_value()),
            "scheduleImpact": lambda n : setattr(self, 'schedule_impact', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithResponsePatchResponse_rfi_status)),
            "suggestedAnswer": lambda n : setattr(self, 'suggested_answer', n.get_str_value()),
            "syncToken": lambda n : setattr(self, 'sync_token', n.get_str_value()),
            "syncVersion": lambda n : setattr(self, 'sync_version', n.get_float_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "virtualFolderUrn": lambda n : setattr(self, 'virtual_folder_urn', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(WithResponsePatchResponse_rfi_watchers)),
            "workflowType": lambda n : setattr(self, 'workflow_type', n.get_enum_value(WithResponsePatchResponse_rfi_workflowType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("answeredAt", self.answered_at)
        writer.write_str_value("answeredBy", self.answered_by)
        writer.write_collection_of_object_values("architects", self.architects)
        writer.write_collection_of_object_values("assignedTo", self.assigned_to)
        writer.write_bool_value("bridgeSyncOutdated", self.bridge_sync_outdated)
        writer.write_bool_value("bridgedSource", self.bridged_source)
        writer.write_bool_value("bridgedTarget", self.bridged_target)
        writer.write_collection_of_primitive_values("category", self.category)
        writer.write_datetime_value("closedAt", self.closed_at)
        writer.write_str_value("closedBy", self.closed_by)
        writer.write_collection_of_object_values("coReviewers", self.co_reviewers)
        writer.write_int_value("commentsCount", self.comments_count)
        writer.write_str_value("constructionManagerId", self.construction_manager_id)
        writer.write_str_value("containerId", self.container_id)
        writer.write_str_value("costImpact", self.cost_impact)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_str_value("customIdentifier", self.custom_identifier)
        writer.write_collection_of_primitive_values("discipline", self.discipline)
        writer.write_collection_of_object_values("draftResponses", self.draft_responses)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_str_value("id", self.id)
        writer.write_str_value("locationDescription", self.location_description)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("managerId", self.manager_id)
        writer.write_int_value("maxAssignees", self.max_assignees)
        writer.write_str_value("officialResponse", self.official_response)
        writer.write_collection_of_object_values("officialResponseActors", self.official_response_actors)
        writer.write_bool_value("officialResponseEditByManagerState", self.official_response_edit_by_manager_state)
        writer.write_enum_value("officialResponseStatus", self.official_response_status)
        writer.write_object_value("permittedActions", self.permitted_actions)
        writer.write_enum_value("previousStatus", self.previous_status)
        writer.write_str_value("priority", self.priority)
        writer.write_str_value("projectId", self.project_id)
        writer.write_str_value("question", self.question)
        writer.write_str_value("reference", self.reference)
        writer.write_datetime_value("respondedAt", self.responded_at)
        writer.write_str_value("respondedBy", self.responded_by)
        writer.write_collection_of_object_values("responses", self.responses)
        writer.write_collection_of_object_values("reviewers", self.reviewers)
        writer.write_uuid_value("rfiTypeId", self.rfi_type_id)
        writer.write_str_value("scheduleImpact", self.schedule_impact)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("suggestedAnswer", self.suggested_answer)
        writer.write_str_value("syncToken", self.sync_token)
        writer.write_float_value("syncVersion", self.sync_version)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("virtualFolderUrn", self.virtual_folder_urn)
        writer.write_collection_of_object_values("watchers", self.watchers)
        writer.write_enum_value("workflowType", self.workflow_type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ManifestGetResponse_prev_databases(Parsable):
    # OSS path for the attributes file.
    attributes: Optional[str] = None
    # DB ID.
    id: Optional[str] = None
    # OSS path for the IDs file.
    ids: Optional[str] = None
    # OSS path for the mapping file.
    mapping: Optional[str] = None
    # OSS path for the offsets file.
    offsets: Optional[str] = None
    # OSS path for the values file.
    values: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ManifestGetResponse_prev_databases:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ManifestGetResponse_prev_databases
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ManifestGetResponse_prev_databases()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "attributes": lambda n : setattr(self, 'attributes', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "ids": lambda n : setattr(self, 'ids', n.get_str_value()),
            "mapping": lambda n : setattr(self, 'mapping', n.get_str_value()),
            "offsets": lambda n : setattr(self, 'offsets', n.get_str_value()),
            "values": lambda n : setattr(self, 'values', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("attributes", self.attributes)
        writer.write_str_value("id", self.id)
        writer.write_str_value("ids", self.ids)
        writer.write_str_value("mapping", self.mapping)
        writer.write_str_value("offsets", self.offsets)
        writer.write_str_value("values", self.values)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .items.items_request_builder import ItemsRequestBuilder
    from .items_next_custom_identifier.items_next_custom_identifier_request_builder import ItemsNextCustomIdentifierRequestBuilder
    from .items_validate_custom_identifier.items_validate_custom_identifier_request_builder import ItemsValidateCustomIdentifierRequestBuilder
    from .item_types.item_types_request_builder import ItemTypesRequestBuilder
    from .metadata.metadata_request_builder import MetadataRequestBuilder
    from .packages.packages_request_builder import PackagesRequestBuilder
    from .responses.responses_request_builder import ResponsesRequestBuilder
    from .settings.settings_request_builder import SettingsRequestBuilder
    from .specs.specs_request_builder import SpecsRequestBuilder
    from .templates.templates_request_builder import TemplatesRequestBuilder
    from .users.users_request_builder import UsersRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}", path_parameters)
    
    @property
    def item_types(self) -> ItemTypesRequestBuilder:
        """
        The itemTypes property
        """
        from .item_types.item_types_request_builder import ItemTypesRequestBuilder

        return ItemTypesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def items(self) -> ItemsRequestBuilder:
        """
        The items property
        """
        from .items.items_request_builder import ItemsRequestBuilder

        return ItemsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def items_next_custom_identifier(self) -> ItemsNextCustomIdentifierRequestBuilder:
        """
        The itemsNextCustomIdentifier property
        """
        from .items_next_custom_identifier.items_next_custom_identifier_request_builder import ItemsNextCustomIdentifierRequestBuilder

        return ItemsNextCustomIdentifierRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def items_validate_custom_identifier(self) -> ItemsValidateCustomIdentifierRequestBuilder:
        """
        The itemsValidateCustomIdentifier property
        """
        from .items_validate_custom_identifier.items_validate_custom_identifier_request_builder import ItemsValidateCustomIdentifierRequestBuilder

        return ItemsValidateCustomIdentifierRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def metadata(self) -> MetadataRequestBuilder:
        """
        The metadata property
        """
        from .metadata.metadata_request_builder import MetadataRequestBuilder

        return MetadataRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def packages(self) -> PackagesRequestBuilder:
        """
        The packages property
        """
        from .packages.packages_request_builder import PackagesRequestBuilder

        return PackagesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def responses(self) -> ResponsesRequestBuilder:
        """
        The responses property
        """
        from .responses.responses_request_builder import ResponsesRequestBuilder

        return ResponsesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def settings(self) -> SettingsRequestBuilder:
        """
        The settings property
        """
        from .settings.settings_request_builder import SettingsRequestBuilder

        return SettingsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def specs(self) -> SpecsRequestBuilder:
        """
        The specs property
        """
        from .specs.specs_request_builder import SpecsRequestBuilder

        return SpecsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def templates(self) -> TemplatesRequestBuilder:
        """
        The templates property
        """
        from .templates.templates_request_builder import TemplatesRequestBuilder

        return TemplatesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def users(self) -> UsersRequestBuilder:
        """
        The users property
        """
        from .users.users_request_builder import UsersRequestBuilder

        return UsersRequestBuilder(self.request_adapter, self.path_parameters)
    




from enum import Enum

class ReviewSheetsGetResponse_results_processingState(str, Enum):
    PROCESSING = "PROCESSING",
    AUDITING = "AUDITING",
    ROTATING = "ROTATING",
    READY = "READY",
    FAILED = "FAILED",
    PUBLISHING = "PUBLISHING",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .versions_get_response_results_approve_status import VersionsGetResponse_results_approveStatus
    from .versions_get_response_results_review_content import VersionsGetResponse_results_reviewContent

@dataclass
class VersionsGetResponse_results(Parsable):
    # The approval status assigned to the file during the review.
    approve_status: Optional[VersionsGetResponse_results_approveStatus] = None
    # The URN of the version copied to the target folder after the file was approved.This field is only present if the workflow includes a copy action and the file was approved.
    copied_file_version_urn: Optional[str] = None
    # The URN of the file item this version belongs to.
    item_urn: Optional[str] = None
    # The name of the resulting file version.
    name: Optional[str] = None
    # Review-specific metadata related to the file version.
    review_content: Optional[VersionsGetResponse_results_reviewContent] = None
    # The URN of the file version currently under review.This value is used when retrieving the full approval and review history of a specific version.See `GET approval-statuses </en/docs/acc/v1/reference/http/reviews-getversionapprovalstatuses-GET/>`_ for more details.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VersionsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VersionsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VersionsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .versions_get_response_results_approve_status import VersionsGetResponse_results_approveStatus
        from .versions_get_response_results_review_content import VersionsGetResponse_results_reviewContent

        from .versions_get_response_results_approve_status import VersionsGetResponse_results_approveStatus
        from .versions_get_response_results_review_content import VersionsGetResponse_results_reviewContent

        fields: dict[str, Callable[[Any], None]] = {
            "approveStatus": lambda n : setattr(self, 'approve_status', n.get_object_value(VersionsGetResponse_results_approveStatus)),
            "copiedFileVersionUrn": lambda n : setattr(self, 'copied_file_version_urn', n.get_str_value()),
            "itemUrn": lambda n : setattr(self, 'item_urn', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "reviewContent": lambda n : setattr(self, 'review_content', n.get_object_value(VersionsGetResponse_results_reviewContent)),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("approveStatus", self.approve_status)
        writer.write_str_value("copiedFileVersionUrn", self.copied_file_version_urn)
        writer.write_str_value("itemUrn", self.item_urn)
        writer.write_str_value("name", self.name)
        writer.write_object_value("reviewContent", self.review_content)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .views_get_response_model_set_views_definition import ViewsGetResponse_modelSetViews_definition

@dataclass
class ViewsGetResponse_modelSetViews(Parsable):
    # The ID of the user or service that created the view.
    created_by: Optional[str] = None
    # The date and time that the view was created.
    created_time: Optional[datetime.datetime] = None
    # The definition of models in a model set view, which is used to track the same models through time. Min items: 1 Max items: 1000.
    definition: Optional[list[ViewsGetResponse_modelSetViews_definition]] = None
    # The description of the model set view. Min length: 1 Max length: 1024.
    description: Optional[str] = None
    # Determines whether the view is only accessible to its creator.
    is_private: Optional[bool] = None
    # The ID of the user or service that last modified the view.
    modified_by: Optional[str] = None
    # The date and time that the view was last modified.
    modified_time: Optional[datetime.datetime] = None
    # The name of the model set view. Min length: 1 Max length: 64.
    name: Optional[str] = None
    # The GUID that uniquely identifies the view.
    view_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ViewsGetResponse_modelSetViews:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ViewsGetResponse_modelSetViews
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ViewsGetResponse_modelSetViews()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .views_get_response_model_set_views_definition import ViewsGetResponse_modelSetViews_definition

        from .views_get_response_model_set_views_definition import ViewsGetResponse_modelSetViews_definition

        fields: dict[str, Callable[[Any], None]] = {
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdTime": lambda n : setattr(self, 'created_time', n.get_datetime_value()),
            "definition": lambda n : setattr(self, 'definition', n.get_collection_of_object_values(ViewsGetResponse_modelSetViews_definition)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "isPrivate": lambda n : setattr(self, 'is_private', n.get_bool_value()),
            "modifiedBy": lambda n : setattr(self, 'modified_by', n.get_str_value()),
            "modifiedTime": lambda n : setattr(self, 'modified_time', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "viewId": lambda n : setattr(self, 'view_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("createdTime", self.created_time)
        writer.write_collection_of_object_values("definition", self.definition)
        writer.write_str_value("description", self.description)
        writer.write_bool_value("isPrivate", self.is_private)
        writer.write_str_value("modifiedBy", self.modified_by)
        writer.write_datetime_value("modifiedTime", self.modified_time)
        writer.write_str_value("name", self.name)
        writer.write_uuid_value("viewId", self.view_id)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_response_patch_response_rfi_draft_responses_state import WithResponsePatchResponse_rfi_draftResponses_state
    from .with_response_patch_response_rfi_draft_responses_status import WithResponsePatchResponse_rfi_draftResponses_status

@dataclass
class WithResponsePatchResponse_rfi_draftResponses(Parsable):
    # The date and time the response was created, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the assigned user.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.To retrieve the list of available assignees, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ and check the ``manager`` and ``projectReviewer`` fields.
    created_by: Optional[str] = None
    # The date and time the response was deleted, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    deleted_at: Optional[datetime.datetime] = None
    # The unique identifier of the response.
    id: Optional[UUID] = None
    # Indicates whether the response can be edited.``true``: The response is editable.``false`` (default): The response cannot be edited.
    is_editable: Optional[bool] = None
    # The Autodesk ID of the assigned user.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.To retrieve the list of available assignees, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ and check the ``manager`` and ``projectReviewer`` fields.
    on_behalf: Optional[str] = None
    # The ID of the RFI associated with this response.
    rfi_id: Optional[str] = None
    # The state of the response. Possible values:- ``draft``: The response was returned to the reviewer for changes.- ``submitted``: The reviewer submitted a finalized response.
    state: Optional[WithResponsePatchResponse_rfi_draftResponses_state] = None
    # Indicates whether the response was accepted (``answered``) or declined (``rejected``).Possible values: ``answered``, ``rejected``
    status: Optional[WithResponsePatchResponse_rfi_draftResponses_status] = None
    # The body of the response.
    text: Optional[str] = None
    # The date and time the response was updated, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who updated the response. To find the name of the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchResponse_rfi_draftResponses:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchResponse_rfi_draftResponses
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchResponse_rfi_draftResponses()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_response_rfi_draft_responses_state import WithResponsePatchResponse_rfi_draftResponses_state
        from .with_response_patch_response_rfi_draft_responses_status import WithResponsePatchResponse_rfi_draftResponses_status

        from .with_response_patch_response_rfi_draft_responses_state import WithResponsePatchResponse_rfi_draftResponses_state
        from .with_response_patch_response_rfi_draft_responses_status import WithResponsePatchResponse_rfi_draftResponses_status

        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isEditable": lambda n : setattr(self, 'is_editable', n.get_bool_value()),
            "onBehalf": lambda n : setattr(self, 'on_behalf', n.get_str_value()),
            "rfiId": lambda n : setattr(self, 'rfi_id', n.get_str_value()),
            "state": lambda n : setattr(self, 'state', n.get_enum_value(WithResponsePatchResponse_rfi_draftResponses_state)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithResponsePatchResponse_rfi_draftResponses_status)),
            "text": lambda n : setattr(self, 'text', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isEditable", self.is_editable)
        writer.write_str_value("onBehalf", self.on_behalf)
        writer.write_str_value("rfiId", self.rfi_id)
        writer.write_enum_value("state", self.state)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("text", self.text)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ClosedPostResponse_clashData_clashes(Parsable):
    # The clash instance index ID. Min items: 2 Max items: 2.
    clash: Optional[list[int]] = None
    # The clash distance.
    dist: Optional[int] = None
    # The clash index ID.
    id: Optional[int] = None
    # The status of the clash.
    status: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClosedPostResponse_clashData_clashes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClosedPostResponse_clashData_clashes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClosedPostResponse_clashData_clashes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "clash": lambda n : setattr(self, 'clash', n.get_collection_of_primitive_values(int)),
            "dist": lambda n : setattr(self, 'dist', n.get_int_value()),
            "id": lambda n : setattr(self, 'id', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("clash", self.clash)
        writer.write_int_value("dist", self.dist)
        writer.write_int_value("id", self.id)
        writer.write_str_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .comments_get_response import CommentsGetResponse
    from .comments_post_request_body import CommentsPostRequestBody
    from .comments_post_response import CommentsPostResponse

class CommentsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/issues/v1/projects/{projectId}/issues/{issueId}/comments
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new CommentsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/issues/v1/projects/{projectId}/issues/{issueId}/comments{?limit*,offset*,sortBy*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CommentsRequestBuilderGetQueryParameters]] = None) -> Optional[CommentsGetResponse]:
        """
        Get all the comments for a specific issue.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CommentsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .comments_get_response import CommentsGetResponse

        return await self.request_adapter.send_async(request_info, CommentsGetResponse, None)
    
    async def post(self,body: CommentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[CommentsPostResponse]:
        """
        Creates a new comment under a specific issue.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CommentsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .comments_post_response import CommentsPostResponse

        return await self.request_adapter.send_async(request_info, CommentsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CommentsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Get all the comments for a specific issue.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: CommentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new comment under a specific issue.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> CommentsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CommentsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CommentsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CommentsRequestBuilderGetQueryParameters():
        """
        Get all the comments for a specific issue.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "sort_by":
                return "sortBy"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Add ``limit=20`` to limit the results count (together with the offset to support pagination).
        limit: Optional[int] = None

        # Add ``offset=20`` to get partial results (together with the limit to support pagination).
        offset: Optional[int] = None

        # Sort issue comments by specified fields. Separate multiple values with commas. To sort in descending order add a ``-`` (minus sign) before the sort criteria. For example: ``sortBy=createdAt,-updatedAt``. Possible values: ``createdAt``, ``updatedAt``, ``createdBy``.
        sort_by: Optional[list[str]] = None

    
    @dataclass
    class CommentsRequestBuilderGetRequestConfiguration(RequestConfiguration[CommentsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class CommentsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class GetSortQueryParameterType(str, Enum):
    SequenceIdDesc = "sequenceId desc",
    Asc = "asc",
    SortFieldAscOrDesc = "sort=<field> [asc or desc]",
    Status = "status",
    SequenceId = "sequenceId",
    Title = "title",
    SentByName = "sentByName",
    CreatedAt = "createdAt",
    DocumentsCount = "documentsCount",
    SortStatusAsc = "sort=status asc",
    SortCreatedAtDesc = "sort=createdAt desc",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_export_get_response_result import WithExportGetResponse_result
    from .with_export_get_response_status import WithExportGetResponse_status

@dataclass
class WithExportGetResponse(Parsable):
    # The ID of the sheets export job.
    id: Optional[UUID] = None
    # The result of a completed export job.- If the ``status`` is ``successful``, a downloadable signed URL will be included in the ``result.output`` object.- If the ``status`` value is ``failed`` (e.g., because some files were deleted), the ``result.error`` object will include details of the error.
    result: Optional[WithExportGetResponse_result] = None
    # The status of the sheets export job.Possible values: ``successful``, ``processing``, ``failed``
    status: Optional[WithExportGetResponse_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithExportGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithExportGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithExportGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_export_get_response_result import WithExportGetResponse_result
        from .with_export_get_response_status import WithExportGetResponse_status

        from .with_export_get_response_result import WithExportGetResponse_result
        from .with_export_get_response_status import WithExportGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "result": lambda n : setattr(self, 'result', n.get_object_value(WithExportGetResponse_result)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithExportGetResponse_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("id", self.id)
        writer.write_object_value("result", self.result)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .tests_get_response_tests_status import TestsGetResponse_tests_status

@dataclass
class TestsGetResponse_tests(Parsable):
    # The date and time that the clash test was completed.
    completed_on: Optional[datetime.datetime] = None
    # The GUID that uniquely identifies the clash test.
    id: Optional[UUID] = None
    # The GUID that uniquely identifies the model set associated with the clash test.
    model_set_id: Optional[UUID] = None
    # The version number of the model set associated with the clash test.
    model_set_version: Optional[int] = None
    # The status of the clash test. If the status is ``Success``, the results of the clash test are available for use. Possible values: ``Pending``, ``Processing``, ``Success``, ``Failed``.
    status: Optional[TestsGetResponse_tests_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TestsGetResponse_tests:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TestsGetResponse_tests
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TestsGetResponse_tests()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .tests_get_response_tests_status import TestsGetResponse_tests_status

        from .tests_get_response_tests_status import TestsGetResponse_tests_status

        fields: dict[str, Callable[[Any], None]] = {
            "completedOn": lambda n : setattr(self, 'completed_on', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "modelSetId": lambda n : setattr(self, 'model_set_id', n.get_uuid_value()),
            "modelSetVersion": lambda n : setattr(self, 'model_set_version', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(TestsGetResponse_tests_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("completedOn", self.completed_on)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("modelSetId", self.model_set_id)
        writer.write_int_value("modelSetVersion", self.model_set_version)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .attachments.attachments_request_builder import AttachmentsRequestBuilder
    from .attachments_batch_create.attachments_batch_create_request_builder import AttachmentsBatchCreateRequestBuilder
    from .attachment_folders.attachment_folders_request_builder import AttachmentFoldersRequestBuilder
    from .budgets.budgets_request_builder import BudgetsRequestBuilder
    from .budgets_contracts_link.budgets_contracts_link_request_builder import BudgetsContractsLinkRequestBuilder
    from .budgets_import.budgets_import_request_builder import BudgetsImportRequestBuilder
    from .change_orders.change_orders_request_builder import ChangeOrdersRequestBuilder
    from .contracts.contracts_request_builder import ContractsRequestBuilder
    from .cost_items.cost_items_request_builder import CostItemsRequestBuilder
    from .cost_items_attach.cost_items_attach_request_builder import CostItemsAttachRequestBuilder
    from .cost_items_batch_create.cost_items_batch_create_request_builder import CostItemsBatchCreateRequestBuilder
    from .cost_items_detach.cost_items_detach_request_builder import CostItemsDetachRequestBuilder
    from .documents.documents_request_builder import DocumentsRequestBuilder
    from .expenses.expenses_request_builder import ExpensesRequestBuilder
    from .main_contracts.main_contracts_request_builder import MainContractsRequestBuilder
    from .payments.payments_request_builder import PaymentsRequestBuilder
    from .payment_items.payment_items_request_builder import PaymentItemsRequestBuilder
    from .performance_tracking_items.performance_tracking_items_request_builder import PerformanceTrackingItemsRequestBuilder
    from .performance_tracking_item_instances.performance_tracking_item_instances_request_builder import PerformanceTrackingItemInstancesRequestBuilder
    from .properties.properties_request_builder import PropertiesRequestBuilder
    from .property_values_batch_update.property_values_batch_update_request_builder import PropertyValuesBatchUpdateRequestBuilder
    from .schedule_of_values.schedule_of_values_request_builder import ScheduleOfValuesRequestBuilder
    from .segments.segments_request_builder import SegmentsRequestBuilder
    from .segment_values.segment_values_request_builder import SegmentValuesRequestBuilder
    from .taxes.taxes_request_builder import TaxesRequestBuilder
    from .templates.templates_request_builder import TemplatesRequestBuilder
    from .time_sheets.time_sheets_request_builder import TimeSheetsRequestBuilder
    from .workflows.workflows_request_builder import WorkflowsRequestBuilder

class WithContainerItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithContainerItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}", path_parameters)
    
    @property
    def attachment_folders(self) -> AttachmentFoldersRequestBuilder:
        """
        The attachmentFolders property
        """
        from .attachment_folders.attachment_folders_request_builder import AttachmentFoldersRequestBuilder

        return AttachmentFoldersRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def attachments(self) -> AttachmentsRequestBuilder:
        """
        The attachments property
        """
        from .attachments.attachments_request_builder import AttachmentsRequestBuilder

        return AttachmentsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def attachments_batch_create(self) -> AttachmentsBatchCreateRequestBuilder:
        """
        The attachmentsBatchCreate property
        """
        from .attachments_batch_create.attachments_batch_create_request_builder import AttachmentsBatchCreateRequestBuilder

        return AttachmentsBatchCreateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def budgets(self) -> BudgetsRequestBuilder:
        """
        The budgets property
        """
        from .budgets.budgets_request_builder import BudgetsRequestBuilder

        return BudgetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def budgets_contracts_link(self) -> BudgetsContractsLinkRequestBuilder:
        """
        The budgetsContractsLink property
        """
        from .budgets_contracts_link.budgets_contracts_link_request_builder import BudgetsContractsLinkRequestBuilder

        return BudgetsContractsLinkRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def budgets_import(self) -> BudgetsImportRequestBuilder:
        """
        The budgetsImport property
        """
        from .budgets_import.budgets_import_request_builder import BudgetsImportRequestBuilder

        return BudgetsImportRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def change_orders(self) -> ChangeOrdersRequestBuilder:
        """
        The changeOrders property
        """
        from .change_orders.change_orders_request_builder import ChangeOrdersRequestBuilder

        return ChangeOrdersRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def contracts(self) -> ContractsRequestBuilder:
        """
        The contracts property
        """
        from .contracts.contracts_request_builder import ContractsRequestBuilder

        return ContractsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def cost_items(self) -> CostItemsRequestBuilder:
        """
        The costItems property
        """
        from .cost_items.cost_items_request_builder import CostItemsRequestBuilder

        return CostItemsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def cost_items_attach(self) -> CostItemsAttachRequestBuilder:
        """
        The costItemsAttach property
        """
        from .cost_items_attach.cost_items_attach_request_builder import CostItemsAttachRequestBuilder

        return CostItemsAttachRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def cost_items_batch_create(self) -> CostItemsBatchCreateRequestBuilder:
        """
        The costItemsBatchCreate property
        """
        from .cost_items_batch_create.cost_items_batch_create_request_builder import CostItemsBatchCreateRequestBuilder

        return CostItemsBatchCreateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def cost_items_detach(self) -> CostItemsDetachRequestBuilder:
        """
        The costItemsDetach property
        """
        from .cost_items_detach.cost_items_detach_request_builder import CostItemsDetachRequestBuilder

        return CostItemsDetachRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def documents(self) -> DocumentsRequestBuilder:
        """
        The documents property
        """
        from .documents.documents_request_builder import DocumentsRequestBuilder

        return DocumentsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def expenses(self) -> ExpensesRequestBuilder:
        """
        The expenses property
        """
        from .expenses.expenses_request_builder import ExpensesRequestBuilder

        return ExpensesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def main_contracts(self) -> MainContractsRequestBuilder:
        """
        The mainContracts property
        """
        from .main_contracts.main_contracts_request_builder import MainContractsRequestBuilder

        return MainContractsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def payment_items(self) -> PaymentItemsRequestBuilder:
        """
        The paymentItems property
        """
        from .payment_items.payment_items_request_builder import PaymentItemsRequestBuilder

        return PaymentItemsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def payments(self) -> PaymentsRequestBuilder:
        """
        The payments property
        """
        from .payments.payments_request_builder import PaymentsRequestBuilder

        return PaymentsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def performance_tracking_item_instances(self) -> PerformanceTrackingItemInstancesRequestBuilder:
        """
        The performanceTrackingItemInstances property
        """
        from .performance_tracking_item_instances.performance_tracking_item_instances_request_builder import PerformanceTrackingItemInstancesRequestBuilder

        return PerformanceTrackingItemInstancesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def performance_tracking_items(self) -> PerformanceTrackingItemsRequestBuilder:
        """
        The performanceTrackingItems property
        """
        from .performance_tracking_items.performance_tracking_items_request_builder import PerformanceTrackingItemsRequestBuilder

        return PerformanceTrackingItemsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def properties(self) -> PropertiesRequestBuilder:
        """
        The properties property
        """
        from .properties.properties_request_builder import PropertiesRequestBuilder

        return PropertiesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def property_values_batch_update(self) -> PropertyValuesBatchUpdateRequestBuilder:
        """
        The propertyValuesBatchUpdate property
        """
        from .property_values_batch_update.property_values_batch_update_request_builder import PropertyValuesBatchUpdateRequestBuilder

        return PropertyValuesBatchUpdateRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def schedule_of_values(self) -> ScheduleOfValuesRequestBuilder:
        """
        The scheduleOfValues property
        """
        from .schedule_of_values.schedule_of_values_request_builder import ScheduleOfValuesRequestBuilder

        return ScheduleOfValuesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def segment_values(self) -> SegmentValuesRequestBuilder:
        """
        The segmentValues property
        """
        from .segment_values.segment_values_request_builder import SegmentValuesRequestBuilder

        return SegmentValuesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def segments(self) -> SegmentsRequestBuilder:
        """
        The segments property
        """
        from .segments.segments_request_builder import SegmentsRequestBuilder

        return SegmentsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def taxes(self) -> TaxesRequestBuilder:
        """
        The taxes property
        """
        from .taxes.taxes_request_builder import TaxesRequestBuilder

        return TaxesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def templates(self) -> TemplatesRequestBuilder:
        """
        The templates property
        """
        from .templates.templates_request_builder import TemplatesRequestBuilder

        return TemplatesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def time_sheets(self) -> TimeSheetsRequestBuilder:
        """
        The timeSheets property
        """
        from .time_sheets.time_sheets_request_builder import TimeSheetsRequestBuilder

        return TimeSheetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def workflows(self) -> WorkflowsRequestBuilder:
        """
        The workflows property
        """
        from .workflows.workflows_request_builder import WorkflowsRequestBuilder

        return WorkflowsRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .responses_post_response_response_attachments_attachments_attachment_type import ResponsesPostResponse_responseAttachments_attachments_attachmentType

@dataclass
class ResponsesPostResponse_responseAttachments_attachments(Parsable):
    # The unique ID of the attachment.
    attachment_id: Optional[UUID] = None
    # The type of the attachment.Possible values: ``rfiResponse``, ``rfiOfficialResponse``, ``bridgeFiles``, ``rfiWebHiddenFiles``.
    attachment_type: Optional[ResponsesPostResponse_responseAttachments_attachments_attachmentType] = None
    # Not relevant
    bubble_urn: Optional[str] = None
    # The ID of the container associated with the attachment.
    container_id: Optional[str] = None
    # The Autodesk ID of the user who added the attachment.
    created_by: Optional[str] = None
    # The name of the user who added the attachment.
    created_by_name: Optional[str] = None
    # The timestamp of the date and time the attachment was created, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    created_on: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who deleted the attachment.
    deleted_by: Optional[str] = None
    # The timestamp of the date and time the attachment was deleted, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    deleted_on: Optional[datetime.datetime] = None
    # The name of the attachment file as it appears in the UI.
    display_name: Optional[str] = None
    # Not relevant
    docs_id: Optional[UUID] = None
    # The ID of the related entity.
    domain_entity_id: Optional[UUID] = None
    # The original name of the uploaded file, including its extension.
    file_name: Optional[str] = None
    # The size of the attachment file in bytes.
    file_size: Optional[int] = None
    # The file extension type for the attachment.
    file_type: Optional[str] = None
    # Not relevant
    is_deleted: Optional[bool] = None
    # Not relevant
    lineage_urn: Optional[str] = None
    # The Autodesk ID of the user who has modified the attachment.
    modified_by: Optional[str] = None
    # The timestamp of the date and time the attachment was modified, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    modified_on: Optional[datetime.datetime] = None
    # The ID of the RFI associated with the attachment.
    rfi_id: Optional[str] = None
    # The storage URN of the attachment file.Use this value to generate a signed URL and download the file via the Data Management API.For more details, see the `Submit RFI Response </en/docs/acc/v1/tutorials/create-rfi-response/>`_ tutorial.
    storage_urn: Optional[str] = None
    # Not relevant
    tip_version_urn: Optional[str] = None
    # The version number of the uploaded file.
    version: Optional[int] = None
    # Not relevant
    version_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResponsesPostResponse_responseAttachments_attachments:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResponsesPostResponse_responseAttachments_attachments
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResponsesPostResponse_responseAttachments_attachments()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .responses_post_response_response_attachments_attachments_attachment_type import ResponsesPostResponse_responseAttachments_attachments_attachmentType

        from .responses_post_response_response_attachments_attachments_attachment_type import ResponsesPostResponse_responseAttachments_attachments_attachmentType

        fields: dict[str, Callable[[Any], None]] = {
            "attachmentId": lambda n : setattr(self, 'attachment_id', n.get_uuid_value()),
            "attachmentType": lambda n : setattr(self, 'attachment_type', n.get_enum_value(ResponsesPostResponse_responseAttachments_attachments_attachmentType)),
            "bubbleUrn": lambda n : setattr(self, 'bubble_urn', n.get_str_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "deletedOn": lambda n : setattr(self, 'deleted_on', n.get_datetime_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "docsId": lambda n : setattr(self, 'docs_id', n.get_uuid_value()),
            "domainEntityId": lambda n : setattr(self, 'domain_entity_id', n.get_uuid_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "fileSize": lambda n : setattr(self, 'file_size', n.get_int_value()),
            "fileType": lambda n : setattr(self, 'file_type', n.get_str_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "lineageUrn": lambda n : setattr(self, 'lineage_urn', n.get_str_value()),
            "modifiedBy": lambda n : setattr(self, 'modified_by', n.get_str_value()),
            "modifiedOn": lambda n : setattr(self, 'modified_on', n.get_datetime_value()),
            "rfiId": lambda n : setattr(self, 'rfi_id', n.get_str_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
            "tipVersionUrn": lambda n : setattr(self, 'tip_version_urn', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
            "versionUrn": lambda n : setattr(self, 'version_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("attachmentId", self.attachment_id)
        writer.write_enum_value("attachmentType", self.attachment_type)
        writer.write_str_value("bubbleUrn", self.bubble_urn)
        writer.write_str_value("containerId", self.container_id)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_datetime_value("deletedOn", self.deleted_on)
        writer.write_str_value("displayName", self.display_name)
        writer.write_uuid_value("docsId", self.docs_id)
        writer.write_uuid_value("domainEntityId", self.domain_entity_id)
        writer.write_str_value("fileName", self.file_name)
        writer.write_int_value("fileSize", self.file_size)
        writer.write_str_value("fileType", self.file_type)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_str_value("lineageUrn", self.lineage_urn)
        writer.write_str_value("modifiedBy", self.modified_by)
        writer.write_datetime_value("modifiedOn", self.modified_on)
        writer.write_str_value("rfiId", self.rfi_id)
        writer.write_str_value("storageUrn", self.storage_urn)
        writer.write_str_value("tipVersionUrn", self.tip_version_urn)
        writer.write_int_value("version", self.version)
        writer.write_str_value("versionUrn", self.version_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .content_views_get_response_results_type import ContentViewsGetResponse_results_type
    from .content_views_get_response_results_view import ContentViewsGetResponse_results_view

@dataclass
class ContentViewsGetResponse_results(Parsable):
    # The content view ID.
    id: Optional[UUID] = None
    # The content view type.Possible values: ``SHEET`` (2D Sheet), ``FILE_MODEL`` (3D Model).
    type: Optional[ContentViewsGetResponse_results_type] = None
    # The content view.
    view: Optional[ContentViewsGetResponse_results_view] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ContentViewsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ContentViewsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ContentViewsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .content_views_get_response_results_type import ContentViewsGetResponse_results_type
        from .content_views_get_response_results_view import ContentViewsGetResponse_results_view

        from .content_views_get_response_results_type import ContentViewsGetResponse_results_type
        from .content_views_get_response_results_view import ContentViewsGetResponse_results_view

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(ContentViewsGetResponse_results_type)),
            "view": lambda n : setattr(self, 'view', n.get_object_value(ContentViewsGetResponse_results_view)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("id", self.id)
        writer.write_enum_value("type", self.type)
        writer.write_object_value("view", self.view)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .values_batch_update_put_request_body_custom_values import ValuesBatchUpdatePutRequestBody_customValues
    from .values_batch_update_put_request_body_tabular_values import ValuesBatchUpdatePutRequestBody_tabularValues

@dataclass
class ValuesBatchUpdatePutRequestBody(Parsable):
    # The list of non-tabular fields.
    custom_values: Optional[list[ValuesBatchUpdatePutRequestBody_customValues]] = None
    # The list of tabular fields.
    tabular_values: Optional[list[ValuesBatchUpdatePutRequestBody_tabularValues]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .values_batch_update_put_request_body_custom_values import ValuesBatchUpdatePutRequestBody_customValues
        from .values_batch_update_put_request_body_tabular_values import ValuesBatchUpdatePutRequestBody_tabularValues

        from .values_batch_update_put_request_body_custom_values import ValuesBatchUpdatePutRequestBody_customValues
        from .values_batch_update_put_request_body_tabular_values import ValuesBatchUpdatePutRequestBody_tabularValues

        fields: dict[str, Callable[[Any], None]] = {
            "customValues": lambda n : setattr(self, 'custom_values', n.get_collection_of_object_values(ValuesBatchUpdatePutRequestBody_customValues)),
            "tabularValues": lambda n : setattr(self, 'tabular_values', n.get_collection_of_object_values(ValuesBatchUpdatePutRequestBody_tabularValues)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("customValues", self.custom_values)
        writer.write_collection_of_object_values("tabularValues", self.tabular_values)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_issues_filter import MeGetResponse_issues_filter
    from .me_get_response_issues_new import MeGetResponse_issues_new

@dataclass
class MeGetResponse_issues(Parsable):
    # Not relevant
    filter: Optional[MeGetResponse_issues_filter] = None
    # If this object appears in the response, it indicates that the user can create and modify issues.
    new: Optional[MeGetResponse_issues_new] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse_issues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse_issues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse_issues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_issues_filter import MeGetResponse_issues_filter
        from .me_get_response_issues_new import MeGetResponse_issues_new

        from .me_get_response_issues_filter import MeGetResponse_issues_filter
        from .me_get_response_issues_new import MeGetResponse_issues_new

        fields: dict[str, Callable[[Any], None]] = {
            "filter": lambda n : setattr(self, 'filter', n.get_object_value(MeGetResponse_issues_filter)),
            "new": lambda n : setattr(self, 'new', n.get_object_value(MeGetResponse_issues_new)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("filter", self.filter)
        writer.write_object_value("new", self.new)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_project_item_request_builder import WithProjectItemRequestBuilder

class ProjectsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/forms/v1/projects
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/forms/v1/projects", path_parameters)
    
    def by_project_id(self,project_id: str) -> WithProjectItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.forms.v1.projects.item collection
        param project_id: The ID of the project.Use the `Data Management API </en/docs/data/v2/>`_ to retrieve the project ID. For more information, see the `Retrieve a Project ID <https://forge.autodesk.com/en/docs/acc/v1/tutorials/getting-started/retrieve-account-and-project-id/>`_ tutorial. You need to convert the project ID into a project ID for the ACC API by removing the ``**b.**`` prefix. For example, a project ID of **b.**a4be0c34a-4ab7 translates to a project ID of a4be0c34a-4ab7.
        Returns: WithProjectItemRequestBuilder
        """
        if project_id is None:
            raise TypeError("project_id cannot be null.")
        from .item.with_project_item_request_builder import WithProjectItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["projectId"] = project_id
        return WithProjectItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .documents_get_response import DocumentsGetResponse
    from .get_association_type_query_parameter_type import GetAssociationTypeQueryParameterType

class DocumentsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/documents
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new DocumentsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/documents?associationId={associationId}&associationType={associationType}{&filter%5Blatest%5D*,filter%5Bsigned%5D*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[DocumentsRequestBuilderGetQueryParameters]] = None) -> Optional[DocumentsGetResponse]:
        """
        Gets generated documents.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[DocumentsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .documents_get_response import DocumentsGetResponse

        return await self.request_adapter.send_async(request_info, DocumentsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[DocumentsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Gets generated documents.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> DocumentsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: DocumentsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return DocumentsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class DocumentsRequestBuilderGetQueryParameters():
        """
        Gets generated documents.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "association_id":
                return "associationId"
            if original_name == "association_type":
                return "associationType"
            if original_name == "filterlatest":
                return "filter%5Blatest%5D"
            if original_name == "filtersigned":
                return "filter%5Bsigned%5D"
            return original_name
        
        # The object ID of the item is associated to. For example, ID of the budget, contract or cost item. Use comma separated string for multiple IDs.
        association_id: Optional[list[str]] = None

        # The type of the item is associated to. Possible values ``Budget``, ``Contract``, ``CostItem``, ``FormInstance``, ``Payment``, ``BudgetPayment``, ``Expense``, ``OCO``, ``SCO``.
        association_type: Optional[GetAssociationTypeQueryParameterType] = None

        # Filter to get only the latest version of a document if it has been generated multiple times. For example, filter[latest]=true. This is deprecated as we'll always response the latest document.
        filterlatest: Optional[bool] = None

        # Filter to get only documents that have been signed. For example, filter[signed]=true. This is deprecated.
        filtersigned: Optional[bool] = None

    
    @dataclass
    class DocumentsRequestBuilderGetRequestConfiguration(RequestConfiguration[DocumentsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .sheets_get_response import SheetsGetResponse

class SheetsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/sheets/v1/projects/{projectId}/sheets
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new SheetsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/sheets/v1/projects/{projectId}/sheets{?collectionId*,currentOnly*,fields*,filter%5Btags%5D*,filter%5BversionSetId%5D*,isDeleted*,limit*,offset*,searchText*,withAllTags*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SheetsRequestBuilderGetQueryParameters]] = None) -> Optional[SheetsGetResponse]:
        """
        Retrieves information about the sheets in a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SheetsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .sheets_get_response import SheetsGetResponse

        return await self.request_adapter.send_async(request_info, SheetsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SheetsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves information about the sheets in a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> SheetsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SheetsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return SheetsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class SheetsRequestBuilderGetQueryParameters():
        """
        Retrieves information about the sheets in a project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "collection_id":
                return "collectionId"
            if original_name == "current_only":
                return "currentOnly"
            if original_name == "filtertags":
                return "filter%5Btags%5D"
            if original_name == "filterversion_set_id":
                return "filter%5BversionSetId%5D"
            if original_name == "is_deleted":
                return "isDeleted"
            if original_name == "search_text":
                return "searchText"
            if original_name == "with_all_tags":
                return "withAllTags"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Filter by sheet collection. If not provided, only results in the ungrouped collection are returned.Possible values:- The UUID of an existing collection: Returns only the results within that collection.- ``*`` Returns results from all collections.
        collection_id: Optional[str] = None

        # - ``true`` to only return versions of the sheets with the newest issuance date (this includes future issuance dates). This is only relevant if you uploaded multiple versions of the same sheet that were assigned the same number identifier. If you do not assign a value to the version set ID filter (``filter[versionSetId]``) this defaults to ``true``.- ``false`` to return all sheets in the project.If you specify a value for the version set ID filter (``filter[versionSetId]``) this defaults to ``false``.If you do not specify a version set, the default is ``true``.
        current_only: Optional[bool] = None

        # Specify which attributes you want to appear in the response. You can specify multiple attributes.For example, to only return the sheet ID and sheet number in the response, use ``fields=id&fields=number``.
        fields: Optional[str] = None

        # Filter sheets by tags. You can filter by multiple tags. For example,``filter[tags]=architecture&filter[tags]=structure`` returns sheets that include both ``architecture`` and ``structure`` tags.Use ``filter[tags]=__UNTAGGED`` to only return sheets without any tags.
        filtertags: Optional[str] = None

        # Filter sheets by the version set ID. If you assign a value to this filter, ``currentOnly`` defaults to ``false``. If you do not assign a value, ``currentOnly`` default to ``true``.
        filterversion_set_id: Optional[str] = None

        # - ``true`` to only return deleted sheets.- ``false`` to only return non-deleted sheets.The default value is ``false``. You cannot retrieve both deleted and non-deleted sheets in one call.
        is_deleted: Optional[bool] = None

        # The number of results to return in the response. Acceptable values: ``1``-``100``. Default value: ``100``.
        limit: Optional[int] = None

        # The starting point for the results, specified by item number. The default value is ``0``. For example, use ``offset=3`` to start the results from the third item.
        offset: Optional[int] = None

        # Search for sheets with the specified text. The API only searches in the ``number`` and ``title`` attributes.You can specify a string or part of a string. The search is case sensitive.You can only use one search string per call. For example, ``searchText=kitchen``.
        search_text: Optional[str] = None

        # - ``true`` to return sheets that contain all search tags.- ``false`` to return sheets that contain at least one tag.If ``filter[tags]=__UNTAGGED`` is already in query parameters, ``withAllTags`` does not work.
        with_all_tags: Optional[bool] = None

    
    @dataclass
    class SheetsRequestBuilderGetRequestConfiguration(RequestConfiguration[SheetsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_form_patch_response_custom_values_toggle_val import WithFormPatchResponse_customValues_toggleVal
    from .with_form_patch_response_custom_values_value_name import WithFormPatchResponse_customValues_valueName

@dataclass
class WithFormPatchResponse_customValues(Parsable):
    # Multi select values.
    array_val: Optional[str] = None
    # A single select value.
    choice_val: Optional[str] = None
    # A date value.
    date_val: Optional[str] = None
    # The unique identifier of the field.
    field_id: Optional[str] = None
    # The field's label or question text.
    item_label: Optional[str] = None
    # Text for the field's notes section.Max length: 8000
    notes: Optional[str] = None
    # A numeric value.
    number_val: Optional[float] = None
    # Name of the section containing this field.
    section_label: Optional[str] = None
    # A signature value (base64 encoded SVG).
    svg_val: Optional[str] = None
    # A text value.
    text_val: Optional[str] = None
    # A boolean like enum value.Possible values: ``Yes``, ``No``, ``False``, ``True``, ``Minus``, ``Plus``, ``Fail``, ``Pass``, ``NA``
    toggle_val: Optional[WithFormPatchResponse_customValues_toggleVal] = None
    # Indicates the type of value used for this item.Possible values: ``textVal``, ``toggleVal``, ``arrayVal``, ``numberVal``, ``choiceVal``, ``dateVal``, ``svgVal``
    value_name: Optional[WithFormPatchResponse_customValues_valueName] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithFormPatchResponse_customValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithFormPatchResponse_customValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithFormPatchResponse_customValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_form_patch_response_custom_values_toggle_val import WithFormPatchResponse_customValues_toggleVal
        from .with_form_patch_response_custom_values_value_name import WithFormPatchResponse_customValues_valueName

        from .with_form_patch_response_custom_values_toggle_val import WithFormPatchResponse_customValues_toggleVal
        from .with_form_patch_response_custom_values_value_name import WithFormPatchResponse_customValues_valueName

        fields: dict[str, Callable[[Any], None]] = {
            "arrayVal": lambda n : setattr(self, 'array_val', n.get_str_value()),
            "choiceVal": lambda n : setattr(self, 'choice_val', n.get_str_value()),
            "dateVal": lambda n : setattr(self, 'date_val', n.get_str_value()),
            "fieldId": lambda n : setattr(self, 'field_id', n.get_str_value()),
            "itemLabel": lambda n : setattr(self, 'item_label', n.get_str_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "numberVal": lambda n : setattr(self, 'number_val', n.get_float_value()),
            "sectionLabel": lambda n : setattr(self, 'section_label', n.get_str_value()),
            "svgVal": lambda n : setattr(self, 'svg_val', n.get_str_value()),
            "textVal": lambda n : setattr(self, 'text_val', n.get_str_value()),
            "toggleVal": lambda n : setattr(self, 'toggle_val', n.get_enum_value(WithFormPatchResponse_customValues_toggleVal)),
            "valueName": lambda n : setattr(self, 'value_name', n.get_enum_value(WithFormPatchResponse_customValues_valueName)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("arrayVal", self.array_val)
        writer.write_str_value("choiceVal", self.choice_val)
        writer.write_str_value("dateVal", self.date_val)
        writer.write_str_value("fieldId", self.field_id)
        writer.write_str_value("itemLabel", self.item_label)
        writer.write_str_value("notes", self.notes)
        writer.write_float_value("numberVal", self.number_val)
        writer.write_str_value("sectionLabel", self.section_label)
        writer.write_str_value("svgVal", self.svg_val)
        writer.write_str_value("textVal", self.text_val)
        writer.write_enum_value("toggleVal", self.toggle_val)
        writer.write_enum_value("valueName", self.value_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .status_step_sets_get_response import StatusStepSetsGetResponse
    from .status_step_sets_post_request_body import StatusStepSetsPostRequestBody
    from .status_step_sets_post_response import StatusStepSetsPostResponse

class StatusStepSetsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v1/projects/{projectId}/status-step-sets
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new StatusStepSetsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v1/projects/{projectId}/status-step-sets{?cursorState*,filter%5BupdatedAt%5D*,includeDeleted*,limit*,renameConflicting*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[StatusStepSetsRequestBuilderGetQueryParameters]] = None) -> Optional[StatusStepSetsGetResponse]:
        """
        GET status-step-sets
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StatusStepSetsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .status_step_sets_get_response import StatusStepSetsGetResponse

        return await self.request_adapter.send_async(request_info, StatusStepSetsGetResponse, None)
    
    async def post(self,body: StatusStepSetsPostRequestBody, request_configuration: Optional[RequestConfiguration[StatusStepSetsRequestBuilderPostQueryParameters]] = None) -> Optional[StatusStepSetsPostResponse]:
        """
        POST status-step-sets
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[StatusStepSetsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .status_step_sets_post_response import StatusStepSetsPostResponse

        return await self.request_adapter.send_async(request_info, StatusStepSetsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[StatusStepSetsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        GET status-step-sets
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: StatusStepSetsPostRequestBody, request_configuration: Optional[RequestConfiguration[StatusStepSetsRequestBuilderPostQueryParameters]] = None) -> RequestInformation:
        """
        POST status-step-sets
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> StatusStepSetsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: StatusStepSetsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return StatusStepSetsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class StatusStepSetsRequestBuilderGetQueryParameters():
        """
        GET status-step-sets
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "cursor_state":
                return "cursorState"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "include_deleted":
                return "includeDeleted"
            if original_name == "limit":
                return "limit"
            return original_name
        
        # An opaque cursor token that identifies where the next page of paginated results should start. It's returned ineach paginated response so that it can be supplied in the next request to continue paginated results. If apaginated response contains no ``cursorState`` value, then there are no further pages to return.Omit this field to initiate a paginated request or to restart pagination.
        cursor_state: Optional[str] = None

        # A string that specifies a date and time or a date and time range at which all returned objects mast have beenupdated. A single date and time takes this format: ``YYYY-MM-DDThh:mm:ss.SSSZ``, A date and time range takes thisformat: ``YYYY-MM-DDThh:mm:ss.SSSZ..YYYY-MM-DDThh:mm:ss.SSSZ``. Range queries can be closed or open in eitherdirection: ``YYYY-MM-DDThh:mm:ss.SSSZ..`` or ``..YYYY-MM-DDThh:mm:ss.SSSZ``.
        filterupdated_at: Optional[str] = None

        # Whether or not soft-deleted object should be included in the response. If ``true``, soft-deleted objects are returned. If ``false``, they are not. The default is ``false``.
        include_deleted: Optional[bool] = None

        # The maximum number of objects that can be returned in a page. A request might return fewer objects than the limit if the Assets service runs out of specified objects to return - at the end of a set of paged results, for example. The maximum limit is ``200``; the default limit is ``25``.
        limit: Optional[int] = None

    
    @dataclass
    class StatusStepSetsRequestBuilderGetRequestConfiguration(RequestConfiguration[StatusStepSetsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class StatusStepSetsRequestBuilderPostQueryParameters():
        """
        POST status-step-sets
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "rename_conflicting":
                return "renameConflicting"
            return original_name
        
        # Specifies that any inactive (deleted) entity with a conflicting name should be automatically renamedto avoid a conflict. Only supported by specific APIs.
        rename_conflicting: Optional[bool] = None

    
    @dataclass
    class StatusStepSetsRequestBuilderPostRequestConfiguration(RequestConfiguration[StatusStepSetsRequestBuilderPostQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .users_import_post_request_body_users_products import UsersImportPostRequestBody_users_products

@dataclass
class UsersImportPostRequestBody_users(Parsable):
    # The ID of the company that the user is representing in the project. To obtain a list of all company IDs associated with a project, call `GET projects/:projectId/companies </en/docs/acc/v1/reference/http/projects-:project_id-companies-GET/>`_.
    company_id: Optional[str] = None
    # The email of the user.Max length: 255
    email: Optional[str] = None
    # The first name of the user.Max length: 255
    first_name: Optional[str] = None
    # The last name of the user.Max length: 255
    last_name: Optional[str] = None
    # Information about the products activated in the specified project for this imported user.
    products: Optional[list[UsersImportPostRequestBody_users_products]] = None
    # A list of the IDs of the user's roles in the project.To obtain role IDs for this parameter, you can call `GET projects/:projectId/users </en/docs/acc/v1/reference/http/admin-projects-projectId-users-GET/>`_ endpoint or `GET projects/:projectId/users/:userId </en/docs/acc/v1/reference/http/admin-projects-projectId-users-userId-GET/>`_ and inspect the ``roleId`` field in the response.
    role_ids: Optional[list[str]] = None
    # Not relevant
    user_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UsersImportPostRequestBody_users:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UsersImportPostRequestBody_users
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UsersImportPostRequestBody_users()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .users_import_post_request_body_users_products import UsersImportPostRequestBody_users_products

        from .users_import_post_request_body_users_products import UsersImportPostRequestBody_users_products

        fields: dict[str, Callable[[Any], None]] = {
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "email": lambda n : setattr(self, 'email', n.get_str_value()),
            "firstName": lambda n : setattr(self, 'first_name', n.get_str_value()),
            "lastName": lambda n : setattr(self, 'last_name', n.get_str_value()),
            "products": lambda n : setattr(self, 'products', n.get_collection_of_object_values(UsersImportPostRequestBody_users_products)),
            "roleIds": lambda n : setattr(self, 'role_ids', n.get_collection_of_primitive_values(str)),
            "userId": lambda n : setattr(self, 'user_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("email", self.email)
        writer.write_str_value("firstName", self.first_name)
        writer.write_str_value("lastName", self.last_name)
        writer.write_collection_of_object_values("products", self.products)
        writer.write_collection_of_primitive_values("roleIds", self.role_ids)
        writer.write_str_value("userId", self.user_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class TimeSheetsPatchRequestBody(Parsable):
    # The code that identifies the budget to which the timesheet belongs. Required if both ``trackingItemInstanceId`` and ``trackingItemInstanceNumber`` are omitted from this request. To find the budget code, call `GET budgets </en/docs/bim360/v1/reference/http/cost-budgets-GET/>`_ and inspect ``results.code`` in the response.
    budget_code: Optional[str] = None
    # The last date of the time period covered by the timesheet. This is also the date that the tracked ``inputQuantity`` and ``outputQuantity`` values are considered to have been reported.
    end_date: Optional[str] = None
    # The hours worked during the time period covered by the timesheet.
    input_quantity: Optional[float] = None
    # The quantity of material used during time period covered by the timesheet.
    output_quantity: Optional[float] = None
    # The first date of the time period covered by the timesheet.
    start_date: Optional[str] = None
    # The ID of the tracking item instance to which the timesheet will belong. Required if both ``trackingItemInstanceNumber`` and ``budgetCode`` are omitted from this request. To find the instance ID, call `GET performance-tracking-item-instances </en/docs/bim360/v1/reference/http/cost-performance-tracking-item-instances-GET/>`_ and inspect ``results.id`` in the response.
    tracking_item_instance_id: Optional[str] = None
    # The user-provided code that represents the tracking item instance to which the timesheet will belong. Required if both ``trackingItemInstanceId`` and ``budgetCode`` are omitted from this request. To find the instance number, call `GET performance-tracking-item-instances </en/docs/bim360/v1/reference/http/cost-performance-tracking-item-instances-GET/>`_ and inspect ``results.number`` in the response.
    tracking_item_instance_number: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TimeSheetsPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TimeSheetsPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TimeSheetsPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "budgetCode": lambda n : setattr(self, 'budget_code', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "outputQuantity": lambda n : setattr(self, 'output_quantity', n.get_float_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "trackingItemInstanceId": lambda n : setattr(self, 'tracking_item_instance_id', n.get_str_value()),
            "trackingItemInstanceNumber": lambda n : setattr(self, 'tracking_item_instance_number', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("budgetCode", self.budget_code)
        writer.write_str_value("endDate", self.end_date)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_float_value("outputQuantity", self.output_quantity)
        writer.write_str_value("startDate", self.start_date)
        writer.write_str_value("trackingItemInstanceId", self.tracking_item_instance_id)
        writer.write_str_value("trackingItemInstanceNumber", self.tracking_item_instance_number)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .resources_get_response_resources_extension import ResourcesGetResponse_resources_extension
    from .resources_get_response_resources_headers import ResourcesGetResponse_resources_headers

@dataclass
class ResourcesGetResponse_resources(Parsable):
    # The file extension used by the clash test resource. Possible values: ``json.gz``, ``sqlite``.
    extension: Optional[ResourcesGetResponse_resources_extension] = None
    # The headers used to retrieve the clash test resource.
    headers: Optional[ResourcesGetResponse_resources_headers] = None
    # The type of the clash test resource.
    type: Optional[str] = None
    # The URL used to retrieve the clash test resource.
    url: Optional[str] = None
    # The time in UTC that the request will stop working.
    valid_until: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResourcesGetResponse_resources:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResourcesGetResponse_resources
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResourcesGetResponse_resources()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .resources_get_response_resources_extension import ResourcesGetResponse_resources_extension
        from .resources_get_response_resources_headers import ResourcesGetResponse_resources_headers

        from .resources_get_response_resources_extension import ResourcesGetResponse_resources_extension
        from .resources_get_response_resources_headers import ResourcesGetResponse_resources_headers

        fields: dict[str, Callable[[Any], None]] = {
            "extension": lambda n : setattr(self, 'extension', n.get_enum_value(ResourcesGetResponse_resources_extension)),
            "headers": lambda n : setattr(self, 'headers', n.get_object_value(ResourcesGetResponse_resources_headers)),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "url": lambda n : setattr(self, 'url', n.get_str_value()),
            "validUntil": lambda n : setattr(self, 'valid_until', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("extension", self.extension)
        writer.write_object_value("headers", self.headers)
        writer.write_str_value("type", self.type)
        writer.write_str_value("url", self.url)
        writer.write_datetime_value("validUntil", self.valid_until)
    

